"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.flags = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const meow_1 = __importDefault(require("meow"));
const chalk_1 = __importDefault(require("chalk"));
const debug_1 = require("debug");
const generator_1 = require("./generator");
const config_1 = require("./config");
const debug = debug_1.debug('api-docs-gen:cli');
exports.flags = {
    output: {
        type: 'string',
        alias: 'o',
        isRequired: false
    },
    config: {
        type: 'string',
        alias: 'c',
        isRequired: false
    },
    'generate-style': {
        type: 'string',
        alias: 'g',
        default: "prefix" /* Prefix */
    },
    'tsdoc-config': {
        type: 'string',
        alias: 't'
    }
};
const cli = meow_1.default(`
  Usage
    $ api-docs-gen <package1.api.json> <package2.api.json> ...

  Options
    --config, -c              configration file
    --output, -o              output dierectory that is markdown contents
    --generate-style, -g      document generating style, default 'prefix'
                              'prefix': be able to separated with each package name
                              'directory': be able to separated with each package directory
    --tsdoc-config, -t        tsdoc configration file

  Examples
    $ api-docs-gen package1.api.json
    $ api-docs-gen package1.api.json --output ./docs
    $ api-docs-gen package1.api.json --config docsgen.config.js
    $ api-docs-gen package1.api.json package2.api.json --generate-style directory
    $ api-docs-gen package1.api.json --tsdoc-config ./tsdoc.json
`, {
    flags: exports.flags
});
debug('flags', exports.flags);
// check input
const input = cli.input;
if (!input.length) {
    cli.showHelp(1);
}
debug('input', input);
// resolve output
const output = cli.flags.output != null ? path_1.default.resolve(cli.flags.output) : process.cwd();
debug(`output`, output);
// mkdir directory, if not exist
try {
    fs_1.default.mkdirSync(output, { recursive: true });
}
catch (e) {
    console.error(chalk_1.default.red(`[api-docs-gen] make output directory error: ${e.message}`));
    process.exit(1);
}
// config
let config = config_1.DefaultConfig;
try {
    config = config_1.resolveConfig(cli.flags.config);
}
catch (e) {
    console.error(chalk_1.default.red(`[api-docs-gen] ${e.message}`));
    process.exit(1);
}
debug(`config`, config);
// generate style
const genStyleFlag = cli.flags['generateStyle'];
const genStyle = ["prefix" /* Prefix */, "directory" /* Directory */].includes(genStyleFlag)
    ? genStyleFlag
    : "prefix" /* Prefix */;
debug('packageDocsStyle', genStyleFlag, genStyle);
// tsdoc configratio
const tsdocConfig = cli.flags['tsdoc-config'];
// run
try {
    ;
    (async () => {
        await generator_1.generate(input, output, {
            style: genStyle,
            config,
            tsdocConfigPath: tsdocConfig != null
                ? path_1.default.resolve(process.cwd(), tsdocConfig)
                : undefined,
            errorOnTSDocConfig: (error) => {
                console.log(chalk_1.default.yellow(`⚠️ Error on TSDoc configration: ${error}`));
            },
            done: (pkgname, filepath) => {
                console.log(chalk_1.default.green(`📦 ${pkgname}: 📝 save ${filepath}`));
            }
        });
    })();
}
catch (e) {
    console.error(chalk_1.default.red(`[api-docs-gen] ${e.message}`));
    process.exit(1);
}
process.on('uncaughtException', err => {
    console.error(chalk_1.default.red(`[api-docs-gen] Uncaught exception: ${err}\n`));
    process.exit(1);
});
process.on('unhandledRejection', (reason, p) => {
    console.error(chalk_1.default.red(`[api-docs-gen] Unhandled rejection at: ${p}, reason: ${reason}`));
    process.exit(1);
});
