"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveConfig = exports.DefaultConfig = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const debug_1 = require("debug");
const resolver_1 = require("./resolver");
const processor_1 = require("./processor");
const debug = debug_1.debug('api-docs-gen:config');
/**
 * Default Configration
 *
 * @remarks
 * Default config for the CLI
 *
 * @public
 */
exports.DefaultConfig = {
    linkReferencer: resolver_1.multi,
    processor: processor_1.multi
};
function resolveConfig(configPath) {
    debug('configPath', configPath);
    const cwd = process.cwd();
    let config;
    let resolvedPath;
    if (configPath) {
        resolvedPath = path_1.default.resolve(cwd, configPath);
        const stat = fs_1.default.statSync(resolvedPath);
        if (!stat.isFile) {
            throw new Error('Not found file');
        }
    }
    else {
        const configPath = path_1.default.resolve(cwd, 'docsgen.config.js');
        try {
            fs_1.default.accessSync(configPath, fs_1.default.constants.F_OK);
            resolvedPath = configPath;
        }
        catch (e) {
            debug(`config access error: ${e.message}`);
        }
    }
    if (!resolvedPath) {
        return exports.DefaultConfig;
    }
    try {
        try {
            config = require(resolvedPath);
            debug('processor importing', resolvedPath, config);
        }
        catch (e) {
            if (!/Cannot use import statement|Unexpected token 'export'/.test(e.message)) {
                throw e;
            }
        }
        if (config) {
            config.linkReferencer = config.linkReferencer || resolver_1.multi;
            return config;
        }
        else {
            return exports.DefaultConfig;
        }
    }
    catch (e) {
        throw new Error(`Failed to load config from ${resolvedPath}: ${e.message}`);
    }
}
exports.resolveConfig = resolveConfig;
