"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const path_1 = __importDefault(require("path"));
const api_extractor_model_1 = require("@microsoft/api-extractor-model");
const debug_1 = require("debug");
const tsdoc_1 = require("./tsdoc");
const utils_1 = require("./utils");
const debug = debug_1.debug('api-docs-gen:generator');
/**
 * Generate API docs
 *
 * @param input - input paths
 * @param output - output api docs full path
 * @param options - optiosn for generate, see the {@link GenerateOptions}
 *
 * @public
 */
async function generate(input, output, options) {
    const { style, config, tsdocConfigPath, done, errorOnTSDocConfig } = options;
    debug(`config`, config);
    debug('style', style);
    debug('tsdocConfig', tsdocConfigPath);
    let customTags = [];
    if (tsdocConfigPath) {
        try {
            const tsdocConfig = tsdoc_1.loadTSDocConfig(tsdocConfigPath);
            customTags = tsdoc_1.mergeTSDocTagDefinition(tsdocConfig.tagDefinitions);
            debug('TSDoc custom tags:', customTags);
        }
        catch (e) {
            debug('error on TSDoc Configration', e.message);
            errorOnTSDocConfig && errorOnTSDocConfig(e.message);
        }
    }
    const apiModel = new api_extractor_model_1.ApiModel();
    for (const target of input) {
        debug(`generate from ${target} ...`);
        const apiPackage = tsdoc_1.loadPackage(target, apiModel);
        let result = config.processor(apiModel, apiPackage, style, config.linkReferencer, customTags);
        if (utils_1.isString(result)) {
            result = [
                {
                    filename: 'api.md',
                    body: result
                }
            ];
        }
        if (!Array.isArray(result)) {
            throw new Error('Not supported processor result type');
        }
        let outputTarget = output;
        if (style === 'directory') {
            outputTarget = path_1.default.resolve(output, apiPackage.displayName);
        }
        for (const { filename, body } of result) {
            const filepath = path_1.default.resolve(outputTarget, style === 'prefix' ? `${apiPackage.displayName}-${filename}` : filename);
            const parsedPath = path_1.default.parse(filepath);
            try {
                await utils_1.mkdir(parsedPath.dir);
            }
            catch (e) {
                throw new Error(`Cannot make '${parsedPath.dir} directory: ${e.message}`);
            }
            await utils_1.writeFile(filepath, body);
            done && done(apiPackage.displayName, filepath);
        }
    }
}
exports.generate = generate;
