"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findCustomTags = exports.getDocSectionContent = exports.buildVariableContent = exports.buildTypeAliasContent = exports.buildClassContent = exports.buildInterfaceContent = exports.buildContentForClassinizable = exports.buildEnumContent = exports.buildFunctionContent = void 0;
const debug_1 = require("debug");
const utils_1 = require("../utils");
const debug = debug_1.debug('api-docs-gen:processor:utils');
function buildFunctionContent(style, model, pkg, resolver, builder, item, customTags, base = 2) {
    builder.pushline(`${'#'.repeat(base)} ${item.displayName}`);
    builder.newline();
    const itemDeclared = item;
    const docs = itemDeclared.tsdocComment;
    if (!docs) {
        return;
    }
    // summary
    if (docs.summarySection) {
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.summarySection, item, style, resolver, customTags));
        builder.newline();
    }
    // signature
    if (itemDeclared.excerptTokens) {
        builder.pushline(`**Signature:**`);
        builder.pushline('```typescript');
        builder.pushline(itemDeclared.excerptTokens.map(token => token.text).join(''));
        builder.pushline('```');
        builder.newline();
    }
    // parameters
    const itemParam = item;
    if (itemParam.parameters) {
        builder.pushline(`${'#'.repeat(base + 1)} Parameters`);
        builder.newline();
        builder.pushline(`| Parameter | Type | Description |`);
        builder.pushline(`| --- | --- | --- |`);
        for (const p of itemParam.parameters) {
            builder.pushline(`| ${p.name} | ${utils_1.escapeText(p.parameterTypeExcerpt.text.trim())} | ${p.tsdocParamBlock && p.tsdocParamBlock.content
                ? getDocSectionContent(model, pkg, 
                // @ts-ignore TODO:
                p.tsdocParamBlock.content, item, style, resolver, customTags)
                : ''} |`);
        }
        builder.newline();
    }
    // returns
    if (docs.returnsBlock) {
        builder.pushline(`${'#'.repeat(base + 1)} Returns`);
        builder.newline();
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.returnsBlock.content, item, style, resolver, customTags));
        builder.newline();
    }
    // throws
    // @ts-ignore TODO:
    const throws = findCustomTags(docs.customBlocks, '@throws');
    if (throws.length > 0) {
        builder.pushline(`${'#'.repeat(base + 1)} Throws`);
        builder.newline();
        for (const t of throws) {
            let text = `${getDocSectionContent(model, pkg, t.content, item, style, resolver, customTags)}`;
            if (throws.length > 1) {
                text = `- ` + text;
            }
            builder.pushline(text);
        }
        builder.newline();
    }
    // remarks
    if (docs.remarksBlock) {
        builder.pushline(`${'#'.repeat(base + 1)} Remarks`);
        builder.newline();
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.remarksBlock.content, item, style, resolver, customTags));
        builder.newline();
    }
    // examples
    // @ts-ignore TODO:
    const examples = findCustomTags(docs.customBlocks, '@example');
    if (examples.length > 0) {
        builder.pushline(`${'#'.repeat(base + 1)} Examples`);
        builder.newline();
        let count = 1;
        for (const e of examples) {
            if (examples.length > 1) {
                builder.pushline(`${'#'.repeat(base + 2)} Example ${count}`);
            }
            builder.pushline(`${getDocSectionContent(model, pkg, e.content, item, style, resolver, customTags)}`);
            builder.newline();
            count++;
        }
        builder.newline();
    }
}
exports.buildFunctionContent = buildFunctionContent;
function buildEnumContent(style, model, pkg, resolver, builder, item, customTags, base = 2) {
    builder.pushline(`${'#'.repeat(base)} ${item.displayName}`);
    builder.newline();
    const itemDeclared = item;
    const docs = itemDeclared.tsdocComment;
    if (!docs) {
        return;
    }
    // summary
    if (docs.summarySection) {
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.summarySection, item, style, resolver, customTags));
        builder.newline();
    }
    // signature
    if (itemDeclared.excerptTokens) {
        builder.pushline(`**Signature:**`);
        builder.pushline('```typescript');
        builder.pushline(itemDeclared.excerptTokens.map(token => token.text).join(''));
        builder.pushline('```');
        builder.newline();
    }
    // members
    if (item.members) {
        builder.pushline(`${'#'.repeat(base + 1)} Members`);
        builder.newline();
        builder.pushline(`| Member | Value| Description |`);
        builder.pushline(`| --- | --- | --- |`);
        for (const m of item.members) {
            const memberEnum = m;
            const memberDeclared = m;
            builder.pushline(`| ${memberEnum.displayName} | ${memberEnum.initializerExcerpt.text} | ${memberDeclared.tsdocComment &&
                memberDeclared.tsdocComment.summarySection
                ? getDocSectionContent(model, pkg, 
                // @ts-ignore TODO:
                memberDeclared.tsdocComment.summarySection, item, style, resolver, customTags)
                : ''} |`);
        }
        builder.newline();
    }
    // remarks
    if (docs.remarksBlock) {
        builder.pushline(`${'#'.repeat(base + 1)} Remarks`);
        builder.newline();
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.remarksBlock.content, item, style, resolver, customTags));
        builder.newline();
    }
    // examples
    // @ts-ignore TODO:
    const examples = findCustomTags(docs.customBlocks, '@example');
    if (examples.length > 0) {
        builder.pushline(`${'#'.repeat(base + 1)} Examples`);
        builder.newline();
        let count = 1;
        for (const e of examples) {
            if (examples.length > 1) {
                builder.pushline(`${'#'.repeat(base + 1)} Example ${count}`);
            }
            builder.pushline(`${getDocSectionContent(model, pkg, e.content, item, style, resolver, customTags)}`);
            builder.newline();
            count++;
        }
        builder.newline();
    }
}
exports.buildEnumContent = buildEnumContent;
function buildContentForClassinizable(style, model, pkg, resolver, builder, item, type, customTags, base = 3) {
    const itemDeclared = item;
    const docs = itemDeclared.tsdocComment;
    if (!docs) {
        return;
    }
    // summary
    if (docs.summarySection) {
        builder.pushline(
        // @ts-ignore
        getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.summarySection, item, style, resolver));
        builder.newline();
    }
    // signature
    if (itemDeclared.excerptTokens) {
        builder.pushline(`**Signature:**`);
        builder.pushline('```typescript');
        builder.pushline(itemDeclared.excerptTokens.map(token => token.text).join(''));
        builder.pushline('```');
        builder.newline();
    }
    // parameters
    const itemParam = item;
    if ((type === 'constrcutor' || type === 'method') && itemParam.parameters) {
        builder.pushline(`*Parameters*`);
        builder.newline();
        builder.pushline(`| Parameter | Type | Description |`);
        builder.pushline(`| --- | --- | --- |`);
        for (const p of itemParam.parameters) {
            builder.pushline(`| ${p.name} | ${utils_1.escapeText(p.parameterTypeExcerpt.text.trim())} | ${p.tsdocParamBlock && p.tsdocParamBlock.content
                ? getDocSectionContent(model, pkg, 
                // @ts-ignore TODO:
                p.tsdocParamBlock.content, item, style, resolver, customTags)
                : ''} |`);
        }
        builder.newline();
    }
    // returns
    if (type === 'method' && docs.returnsBlock) {
        builder.pushline(`${'#'.repeat(base)} Returns`);
        builder.newline();
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.returnsBlock.content, item, style, resolver, customTags));
        builder.newline();
    }
    // throws
    // @ts-ignore TODO:
    const throws = findCustomTags(docs.customBlocks, '@throws');
    if ((type === 'constructor' || type === 'method') && throws.length > 0) {
        builder.pushline(`${'#'.repeat(base)} Throws`);
        builder.newline();
        for (const t of throws) {
            let text = `${getDocSectionContent(model, pkg, t.content, item, style, resolver, customTags)}`;
            if (throws.length > 1) {
                text = `- ` + text;
            }
            builder.pushline(text);
        }
        builder.newline();
    }
    // remarks
    if (docs.remarksBlock) {
        builder.pushline(`${'#'.repeat(base)} Remarks`);
        builder.newline();
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.remarksBlock.content, item, style, resolver, customTags));
        builder.newline();
    }
    // examples
    // @ts-ignore TODO:
    const examples = findCustomTags(docs.customBlocks, '@example');
    if (examples.length > 0) {
        builder.pushline(`${'#'.repeat(base)} Examples`);
        builder.newline();
        let count = 1;
        for (const e of examples) {
            if (examples.length > 1) {
                builder.pushline(`${'#'.repeat(base + 1)} Example ${count}`);
            }
            builder.pushline(`${getDocSectionContent(model, pkg, e.content, item, style, resolver, customTags)}`);
            builder.newline();
            count++;
        }
        builder.newline();
    }
}
exports.buildContentForClassinizable = buildContentForClassinizable;
function buildInterfaceContent(style, model, pkg, resolver, builder, item, customTags, base = 2) {
    builder.pushline(`${'#'.repeat(base)} ${item.displayName}`);
    builder.newline();
    buildContentForClassinizable(style, model, pkg, resolver, builder, item, 'interface', customTags, base + 1);
    builder.newline();
    const methods = item.members.filter(m => m.kind === 'MethodSignature');
    builder.pushline(`${'#'.repeat(base + 1)} Methods`);
    builder.newline();
    for (const method of methods) {
        builder.pushline(`${'#'.repeat(base + 2)} ${method.displayName}`);
        builder.newline();
        buildContentForClassinizable(style, model, pkg, resolver, builder, method, 'method', customTags, base + 1);
    }
    builder.newline();
    const properties = item.members.filter(m => m.kind === 'PropertySignature');
    builder.pushline(`${'#'.repeat(base + 1)} Properties`);
    builder.newline();
    for (const property of properties) {
        builder.pushline(`${'#'.repeat(base + 2)} ${property.displayName}`);
        builder.newline();
        buildContentForClassinizable(style, model, pkg, resolver, builder, property, 'property', customTags, base + 1);
    }
    builder.newline();
}
exports.buildInterfaceContent = buildInterfaceContent;
function buildClassContent(style, model, pkg, resolver, builder, item, customTags, base = 2) {
    builder.pushline(`${'#'.repeat(base)} ${item.displayName}`);
    builder.newline();
    buildContentForClassinizable(style, model, pkg, resolver, builder, item, 'class', customTags);
    builder.newline();
    const [ctor] = item.members.filter(m => m.kind === 'Constructor');
    builder.pushline(`${'#'.repeat(base + 1)} Constructor`);
    builder.newline();
    buildContentForClassinizable(style, model, pkg, resolver, builder, ctor, 'constructor', customTags, base + 1);
    builder.newline();
    const methods = item.members.filter(m => m.kind === 'Method');
    builder.pushline(`${'#'.repeat(base + 1)} Methods`);
    builder.newline();
    for (const method of methods) {
        builder.pushline(`${'#'.repeat(base + 2)} ${method.displayName}`);
        builder.newline();
        buildContentForClassinizable(style, model, pkg, resolver, builder, method, 'method', customTags, base + 1);
    }
    builder.newline();
    const properties = item.members.filter(m => m.kind === 'Property');
    builder.pushline(`${'#'.repeat(base + 1)} Properties`);
    builder.newline();
    for (const property of properties) {
        builder.pushline(`${'#'.repeat(base + 2)} ${property.displayName}`);
        builder.newline();
        buildContentForClassinizable(style, model, pkg, resolver, builder, property, 'property', customTags, base + 1);
    }
    builder.newline();
}
exports.buildClassContent = buildClassContent;
function buildTypeAliasContent(style, model, pkg, resolver, builder, item, customTags, base = 2) {
    builder.pushline(`${'#'.repeat(base)} ${item.displayName}`);
    builder.newline();
    const itemDeclared = item;
    const docs = itemDeclared.tsdocComment;
    if (!docs) {
        return;
    }
    // summary
    if (docs.summarySection) {
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.summarySection, item, style, resolver, customTags));
        builder.newline();
    }
    // signature
    if (itemDeclared.excerptTokens) {
        builder.pushline(`**Signature:**`);
        builder.pushline('```typescript');
        builder.pushline(itemDeclared.excerptTokens.map(token => token.text).join(''));
        builder.pushline('```');
        builder.newline();
    }
    // remarks
    if (docs.remarksBlock) {
        builder.pushline(`${'#'.repeat(base + 1)} Remarks`);
        builder.newline();
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.remarksBlock.content, item, style, resolver, customTags));
        builder.newline();
    }
    // examples
    // @ts-ignore TODO:
    const examples = findCustomTags(docs.customBlocks, '@example');
    if (examples.length > 0) {
        builder.pushline(`${'#'.repeat(base) + 1} Examples`);
        builder.newline();
        let count = 1;
        for (const e of examples) {
            if (examples.length > 1) {
                builder.pushline(`${'#'.repeat(base + 2)} Example ${count}`);
            }
            builder.pushline(`${getDocSectionContent(model, pkg, e.content, item, style, resolver, customTags)}`);
            builder.newline();
            count++;
        }
        builder.newline();
    }
}
exports.buildTypeAliasContent = buildTypeAliasContent;
function buildVariableContent(style, model, pkg, resolver, builder, item, customTags, base = 2) {
    builder.pushline(`${'#'.repeat(base)} ${item.displayName}`);
    builder.newline();
    const itemDeclared = item;
    const docs = itemDeclared.tsdocComment;
    if (!docs) {
        return;
    }
    // summary
    if (docs.summarySection) {
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.summarySection, item, style, resolver, customTags));
        builder.newline();
    }
    // signature
    if (itemDeclared.excerptTokens) {
        builder.pushline(`**Signature:**`);
        builder.pushline('```typescript');
        builder.pushline(itemDeclared.excerptTokens.map(token => token.text).join(''));
        builder.pushline('```');
        builder.newline();
    }
    // remarks
    if (docs.remarksBlock) {
        builder.pushline(`${'#'.repeat(base + 1)} Remarks`);
        builder.newline();
        builder.pushline(getDocSectionContent(model, pkg, 
        // @ts-ignore TODO:
        docs.remarksBlock.content, item, style, resolver, customTags));
        builder.newline();
    }
    // examples
    // @ts-ignore TODO:
    const examples = findCustomTags(docs.customBlocks, '@example');
    if (examples.length > 0) {
        builder.pushline(`${'#'.repeat(base + 1)} Examples`);
        builder.newline();
        let count = 1;
        for (const e of examples) {
            if (examples.length > 1) {
                builder.pushline(`${'#'.repeat(base + 2)} Example ${count}`);
            }
            builder.pushline(`${getDocSectionContent(model, pkg, e.content, item, style, resolver, customTags)}`);
            builder.newline();
            count++;
        }
        builder.newline();
    }
}
exports.buildVariableContent = buildVariableContent;
/**
 * Get DocSection content
 *
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 * @param content - a {@link https://github.com/microsoft/tsdoc/blob/master/tsdoc/src/nodes/DocSection.ts | content}
 * @param contextItem - a context {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | item}
 * @param style - generate style, See the {@link GenerateStyle}
 * @param resolver - {@link ReferenceResolver | resolver} to resolve markdown content references
 *
 * @returns doc section markdown content
 *
 * @public
 */
function getDocSectionContent(model, pkg, content, contextItem, style, resolver, customTags // eslint-disable-line @typescript-eslint/no-unused-vars
) {
    let ret = '';
    for (const n of content.nodes) {
        if (n.kind === "Paragraph" /* Paragraph */) {
            for (const nn of n.getChildNodes()) {
                if (nn.kind === "PlainText" /* PlainText */) {
                    ret += nn.text;
                }
                else if (nn.kind === "CodeSpan" /* CodeSpan */) {
                    ret += `\`${nn.code}\``;
                }
                else if (nn.kind === "LinkTag" /* LinkTag */) {
                    const { linkText, urlDestination, codeDestination } = nn;
                    if (codeDestination) {
                        const result = resolveDeclarationReference(model, codeDestination, contextItem);
                        if (result.resolvedApiItem) {
                            const filepath = resolver(style, result.resolvedApiItem, model, pkg);
                            if (linkText) {
                                const encodedLinkText = utils_1.escapeText(linkText.replace(/\s+/g, ' '));
                                ret += `[${encodedLinkText}](${filepath})`;
                            }
                            else {
                                ret += `[${result.resolvedApiItem.displayName}](${filepath})`;
                            }
                        }
                    }
                    else {
                        if (linkText) {
                            if (urlDestination) {
                                const encodedLinkText = utils_1.escapeText(linkText.replace(/\s+/g, ' '));
                                ret += `[${encodedLinkText}](${urlDestination})`;
                            }
                            else {
                                ret += utils_1.escapeText(linkText.replace(/\s+/g, ' '));
                            }
                        }
                        else {
                            if (urlDestination) {
                                ret += urlDestination;
                            }
                        }
                    }
                }
            }
        }
        else if (n.kind === "FencedCode" /* FencedCode */) {
            const fenced = n;
            ret += `\n`;
            ret += `\`\`\`${fenced.language}\n`;
            ret += fenced.code;
            ret += `\`\`\``;
        }
        else {
            debug(`ignore build doc section: ${n.kind}`);
        }
    }
    return ret;
}
exports.getDocSectionContent = getDocSectionContent;
/**
 * Find custom tags
 *
 * @remarks
 * About custom tags, See the {@link https://github.com/microsoft/tsdoc/issues/21 | this issue}
 *
 * @param customBlocks - target cusotm blocks
 * @param tag - finding target tag
 *
 * @returns found custom blocks
 *
 * @public
 */
function findCustomTags(customBlocks, tag) {
    return customBlocks.filter(x => x.blockTag.tagName === tag);
}
exports.findCustomTags = findCustomTags;
function resolveDeclarationReference(model, reference, item) {
    // @ts-ignore TODO:
    return model.resolveDeclarationReference(reference, item);
}
