"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolve = void 0;
const utils_1 = require("../utils");
/**
 * Resolve the markdown content reference
 *
 * @remarks
 * This reference resolver is used by the {@link multiProcessor | processor} to generate API docs references for separate pieces of markdown content.
 *
 * @param style - generate style, See the {@link GenerateStyle}
 * @param item - a {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | item}
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 *
 * @returns resolved the reference string
 *
 * @public
 */
function resolve(style, item, model, pkg) {
    let baseName = '';
    const pkgName = pkg.displayName;
    for (const hierarchyItem of item.getHierarchy()) {
        const qualifiedName = utils_1.getSafePathFromDisplayName(hierarchyItem.displayName);
        switch (hierarchyItem.kind) {
            case "Model" /* Model */:
            case "EntryPoint" /* EntryPoint */:
            case "Package" /* Package */:
                break;
            case "Enum" /* Enum */:
            case "Function" /* Function */:
            case "Variable" /* Variable */:
            case "TypeAlias" /* TypeAlias */:
            case "Class" /* Class */:
            case "Interface" /* Interface */:
                baseName += `${hierarchyItem.kind.toLowerCase()}#${qualifiedName}`;
                break;
            default:
                baseName += '.' + qualifiedName;
        }
    }
    return style === "prefix" /* Prefix */
        ? `./${pkgName}-${baseName}`
        : `./${baseName}`;
}
exports.resolve = resolve;
