"use strict";
/* eslint-disable @typescript-eslint/no-unused-vars */
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolve = void 0;
const utils_1 = require("../utils");
/**
 * Resolve the markdown content reference
 *
 * @remarks
 * This reference resolver is used by the {@link tocProcessor | processor} to generate a reference specifically for API docs of markdown content with TOC.
 *
 * @param style - generate style, See the {@link GenerateStyle}
 * @param item - a {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | item}
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 *
 * @returns resolved the reference string
 *
 * @public
 */
function resolve(style, item, model, pkg) {
    let baseName = '';
    for (const hierarchyItem of item.getHierarchy()) {
        const qualifiedName = utils_1.getSafePathFromDisplayName(hierarchyItem.displayName);
        switch (hierarchyItem.kind) {
            case "Enum" /* Enum */:
            case "Function" /* Function */:
            case "Variable" /* Variable */:
            case "TypeAlias" /* TypeAlias */:
            case "Class" /* Class */:
            case "Interface" /* Interface */:
                baseName = `#${qualifiedName}`;
                break;
            default:
                break;
        }
    }
    return baseName;
}
exports.resolve = resolve;
/* eslint-enable @typescript-eslint/no-unused-vars */
